/*
 * cancan.h
 * various constants etc
 */

/* CONSTANTS */

#define DEFAULT_EDITOR	"emacs"	/* (environment) variable EDITOR overrides */
#define DEFAULT_PAGER	"more"	/* (environment) variable PAGER overrides */

#define CMDSEP	';'		/* command separator character */
#define DELIM	" ,;:.?!'\"{}"	/* word delimiting characters */

#define MAXMAPLEN 512		/* maximum length of automapped path */
#define WORDSKEPT 256		/* number of words kept for TAB-completion */
#define MAXHIST   128		/* number of history lines kept */

#define FLASHDELAY  500000	/* time of parentheses flash in microsecs */
#define KBD_TIMEOUT  35000	/* timeout for keyboard read in microsecs */
#define SCH_HZ     16		/* scheduled commands delay precision */

#define MAXFDSCAN  32		/* max number of fd to scan in select() */
#define CAPLEN     20		/* max length of a terminal capability */
#define MAXSUBOPT 128		/* max length of telnet suboption string */
#define LINELEN  1024		/* maximum line length */
#define BUFSIZE  2048		/* generic i/o buffer size */

/* linemode flags: */
#define LM_NOECHO 1		/* no local echo */
#define LM_CHAR   2		/* char-by-char mode (no line editing) */

/* attention flags: */
#define ATTN_FLASHBACK	1	/* move cursor back after flashparen() */
#define ATTN_FOOTNOTE	2	/* show/clear the bottom line footnote */
#define ATTN_WINCHSIG	4	/* keep up with window size changes */
#define ATTN_CHILDSIG	8	/* finish editing sessions */

#if !defined(__STDC__) && !defined(const)
#define const
#endif

/* STRUCTURES */

/* from edit.c */
struct defbind_node {
  char label[8];
  char *seq, *name;
  void (*funct)();
};

/* SHARED FUNCTIONS */

/* from cancan.c */
void printver();
void exitcancan();
int execute();
void intrsig();
void suspend_cancan();
int syswarn();
void syserr();
void save_settings();
void record_line();
void to_footnote();
char *str_dup();
void str_assign();
char *get_env();
void set_env();
int is_main_sess();
int find_sess();
int connect_sess();
void disconnect_sess();
void snoop_sess();
void show_connections();

/* from mapping.c */
void remove_steps();
void retrace_steps();
void show_automap();
void add_dir_to_map();

/* from tel.c */
int doconnect();
int read_socket();
void send_term_size();
void send_to_host();
void line_to_host();
void flush_output_buffer();

/* from edit.c */
void default_completions();
void put_word();

/* SHARED VARS */

extern int errno, sys_nerr;
extern char *sys_errlist[];
extern char **environ;

/* from cancan.c */
extern int mp_active;
extern int new_settings;	/* action/alias/etc has changed */
extern int verbose;		/* this is set if cmd is user-typed */
extern int wrap;
extern int speedwalk;
extern int verbatim;
extern int expect_prompt;	/* set when we send_to_host */

extern int linemode;		/* line mode with flags as above (LM_*) */
extern int sockfd;		/* socket file descriptor */
extern int attention;		/* attn flags, should be sig_atomic_t */

extern char promptstr[];	/* prompt string */
extern char edbuf[];		/* input line buffer */
extern int edlen;		/* characters in edbuf */
extern int pos;			/* cursor position in edbuf */
extern char *initstr;
extern void (*last_edit_func)();

extern FILE *capfile;
extern char *prefixstr;
extern char *footnote;
extern char insertbuf[];

extern char *recording_name;

/* from edit.c */
extern const struct defbind_node defbinds[];

#if defined(TERM) && !defined(OMIT_SESSIONS)
#define OMIT_SESSIONS
#endif

#ifdef OMIT_SESSIONS
#define is_main_sess() (1)
#endif

#if defined(USEBCOPY) && !defined(memcpy)
#define memcpy(a, b, c) bcopy((b), (a), (c))
#endif
