/*
 * list.h
 * linked list operations
 */

/* generic linked list node */
typedef struct defnode {
    struct defnode *next;
    char *field;
} defnode;

/* linked list nodes: keep the next pointer first, then string to sort by */
typedef struct aliasnode {
    struct aliasnode *next;
    char *name;
    char *subst;
    int active;
} aliasnode;

/* mml (minimum matchable string length) is used for the sake of efficiency */
typedef struct actionnode {
    struct actionnode *next;
    char *pattern;
    char *commands, *label;
    int active, mml;
} actionnode;

typedef struct marknode {
    struct marknode *next;
    char *pattern, *pos;
    int plen, attrcode;
} marknode;

typedef struct keynode {
    struct keynode *next;	/* next key definition in list */
    char *sequence;		/* escape sequence sent by terminal */
    char *name;			/* key name */
    void (*funct)();		/* function called when key pressed */
    char *call_data;		/* data passed to function */
} keynode;

typedef struct schedulenode {
    struct schedulenode *next;
    char *label;
    char *commands;
    long delta;
} schedulenode;

extern defnode *pager_text;
extern aliasnode *aliases;
extern actionnode *actions;
extern marknode *markers;
extern keynode *keydefs;
extern schedulenode *schedule;

defnode **add_node();
aliasnode **add_aliasnode();
actionnode **add_actionnode();
marknode **add_marknode();
keynode **add_keynode();
schedulenode **add_schedulenode();

schedulenode **reschedule();

defnode **lookup();
aliasnode **lookup_alias();
actionnode **lookup_action();
actionnode **lookup_action_pattern();
marknode **lookup_marker();
keynode **lookup_key();
schedulenode **lookup_schedulenode();

void delete_node();
void delete_aliasnode();
void delete_actionnode();
void delete_marknode();
void delete_keynode();
void delete_schedulenode();

void to_pager();
void delete_pagernode();

/* generic lookup used instead for searching by node's first data field */
#define lookup_alias(name) ((aliasnode **)lookup(name, &aliases))
#define lookup_action_pattern(pattern) ((actionnode **)lookup(pattern, &actions))
#define lookup_marker(pattern) ((marknode **)lookup(pattern, &markers))
#define lookup_schedule(label) ((schedulenode **)lookup(label, &schedule))
